"""
Fakes needed for tests
"""

FAKE_API_TOKEN = "XXXXXXXXXXXXXXXXXXXxxx"
FAKE_RECORD = {
    "record": {
        "id": "123Fake",
    }
}

FAKE_DOMAIN = "some.domain"
FAKE_ZONE_ID = "xyz"
FAKE_RECORD_ID = "zzz"
FAKE_RECORD_NAME = "thisisarecordname"

FAKE_RECORD_RESPONSE = {
    "record": {
        "id": FAKE_RECORD_ID,
        "name": "string",
    }
}

FAKE_RECORDS_RESPONSE_WITH_RECORD = {
    "records": [
        {
            "id": FAKE_RECORD_ID,
            "name": FAKE_RECORD_NAME,
        }
    ]
}

FAKE_RECORDS_RESPONSE_WITHOUT_RECORD = {
    "records": [
        {
            "id": "nottheoneuwant",
            "name": "string",
        }
    ]
}

FAKE_ZONES_RESPONSE_WITH_DOMAIN = {
    "zones": [
        {
            "id": FAKE_ZONE_ID,
            "name": FAKE_DOMAIN,
        }
    ]
}

FAKE_ZONES_RESPONSE_WITHOUT_DOMAIN = {
    "zones": [
        {
            "id": "string",
            "name": "string",
        }
    ]
}



"""DNS Authenticator for Hetzner DNS."""
import tldextract
from certbot.plugins import dns_common
from lexicon.client import Client
from lexicon.config import ConfigResolver

TTL = 60


class Authenticator(dns_common.DNSAuthenticator):
    """DNS Authenticator for Hetzner
    This Authenticator uses the Hetzner DNS API to fulfill a dns-01 challenge.
    """

    description = (
        "Obtain certificates using a DNS TXT record (if you are using Hetzner for DNS)."
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.credentials = None

    @classmethod
    def add_parser_arguments(cls, add, default_propagation_seconds = 60):
        super(Authenticator, cls).add_parser_arguments(
            add, default_propagation_seconds=default_propagation_seconds
        )
        add("credentials", help="Hetzner credentials INI file.")

    def more_info(self):  # pylint: disable=missing-function-docstring
        return (
            "This plugin configures a DNS TXT record to respond to a dns-01 challenge using "
            + "the Hetzner API."
        )

    def _setup_credentials(self):
        self.credentials = self._configure_credentials(
            "credentials",
            "Hetzner credentials INI file",
            {
                "api_token": "Hetzner API Token from 'https://dns.hetzner.com/settings/api-token'",
            },
        )

    @staticmethod
    def _get_zone(domain):
        extract = tldextract.TLDExtract()
        zone_name = extract(domain, include_psl_private_domains=True)
        return '.'.join([zone_name.domain, zone_name.suffix])

    def _perform(self, domain, validation_name, validation):
        with self._get_hetzner_client(domain) as client:
            client.create_record("TXT", self._fqdn_format(validation_name), validation)

    def _cleanup(self, domain, validation_name, validation):
        with self._get_hetzner_client(domain) as client:
            client.delete_record(None, "TXT", self._fqdn_format(validation_name), validation)

    def _get_hetzner_client(self, domain):
        config = ConfigResolver().with_env().with_dict({
            "provider_name": "hetzner",
            "hetzner": {
                "auth_token": self.credentials.conf("api_token")
            },

            "ttl": TTL,
            "domain": self._get_zone(domain),
        })
        return Client(config)

    @staticmethod
    def _fqdn_format(name):
        if not name.endswith("."):
            return f"{name}."
        return name



"""Tests for certbot_dns_hetzner.dns_hetzner."""

import unittest

from unittest import mock

from certbot.compat import os
from certbot.errors import PluginError
from certbot.plugins import dns_test_common
from certbot.plugins.dns_test_common import DOMAIN
from certbot.tests import util as test_util

from certbot_dns_hetzner.fakes import FAKE_API_TOKEN, FAKE_RECORD


patch_display_util = test_util.patch_display_util


class AuthenticatorTest(
    test_util.TempDirTestCase, dns_test_common.BaseAuthenticatorTest
):
    """
    Test for Hetzner DNS Authenticator
    """

    def setUp(self):
        super().setUp()
        from certbot_dns_hetzner.dns_hetzner import Authenticator  # pylint: disable=import-outside-toplevel

        path = os.path.join(self.tempdir, "fake_credentials.ini")
        dns_test_common.write(
            {
                "hetzner_api_token": FAKE_API_TOKEN,
            },
            path,
        )

        super().setUp()
        self.config = mock.MagicMock(
            hetzner_credentials=path, hetzner_propagation_seconds=0
        )  # don't wait during tests

        self.auth = Authenticator(self.config, "hetzner")

        self.mock_client = mock.MagicMock()

        mock_client_wrapper = mock.MagicMock()
        mock_client_wrapper.__enter__ = mock.MagicMock(
            return_value=self.mock_client
        )

        # _get_ispconfig_client | pylint: disable=protected-access
        self.auth._get_hetzner_client = mock.MagicMock(
            return_value=mock_client_wrapper
        )

    @patch_display_util()
    def test_perform(self, _unused_mock_get_utility):
        self.mock_client.create_record.return_value = FAKE_RECORD
        self.auth.perform([self.achall])
        self.mock_client.create_record.assert_called_with(
            "TXT", "_acme-challenge." + DOMAIN + ".", mock.ANY
        )

    def test_perform_but_raises_plugin_error(self):
        self.mock_client.create_record.side_effect = mock.MagicMock(
            side_effect=PluginError()
        )
        self.assertRaises(PluginError, self.auth.perform, [self.achall])
        self.mock_client.create_record.assert_called_with(
            "TXT", "_acme-challenge." + DOMAIN + ".", mock.ANY
        )

    @patch_display_util()
    def test_cleanup(self, _unused_mock_get_utility):
        self.mock_client.create_record.return_value = FAKE_RECORD
        # _attempt_cleanup | pylint: disable=protected-access
        self.auth.perform([self.achall])
        self.auth._attempt_cleanup = True
        self.auth.cleanup([self.achall])

        self.mock_client.delete_record.assert_called_with(
            None, "TXT", "_acme-challenge." + DOMAIN + ".", mock.ANY
        )

    @patch_display_util()
    def test_cleanup_but_raises_plugin_error(self, _unused_mock_get_utility):
        self.mock_client.create_record.return_value = FAKE_RECORD
        self.mock_client.delete_record.side_effect = mock.MagicMock(
            side_effect=PluginError()
        )
        # _attempt_cleanup | pylint: disable=protected-access
        self.auth.perform([self.achall])
        self.auth._attempt_cleanup = True

        self.assertRaises(PluginError, self.auth.cleanup, [self.achall])
        self.mock_client.delete_record.assert_called_with(
            None, "TXT", "_acme-challenge." + DOMAIN + ".", mock.ANY
        )


if __name__ == "__main__":
    unittest.main()  # pragma: no cover
